
package PrintGame;

import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.XMLReader;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import java.io.File;
import java.io.IOException;
import java.net.URL;

//----------------------------------------------------------------------
/**
 * class representing a two dimensional labyrinth
 */

public class LoadLabyrinth 
{

  /** the number of diamonds distributed in the field (set from
   * outside of the class with the appropriate method-calls */
//  protected int num_diamonds_;

  /** the max number of rows
   *  max_row_=rows-1*/
  protected int max_row_;
  /** the max number of cols
   *  max_col_=cols-1*/
  protected int max_col_;

  private URL document_base_;
  protected LabyrinthField labyrinth_field_;
  protected CreateLabyrinth create_labyrinth_;
  protected LabyrinthXMLContentHandler labyrinth_xml_content_handler_;


  private boolean is_style_;
  private String wall_picture_;
  private String way_picture_;
  private String robot_picture_;
  private String diamond_picture_;
  private String start_picture_;
  private String filename_;

//--------------------------------------------------------------------------
// CopyConstructor
	public LoadLabyrinth(LoadLabyrinth lab)
	{
		this.max_row_=lab.max_row_;
		this.max_col_=lab.max_col_;
		this.document_base_=lab.document_base_;
		this.labyrinth_field_=(LabyrinthField) lab.labyrinth_field_.clone();
		this.create_labyrinth_=lab.create_labyrinth_;
		this.labyrinth_xml_content_handler_=lab.labyrinth_xml_content_handler_;
		this.is_style_=lab.is_style_;
		this.wall_picture_=lab.wall_picture_;
		this.way_picture_=lab.way_picture_;
		this.robot_picture_=lab.robot_picture_;
		this.diamond_picture_=lab.diamond_picture_;
		this.start_picture_=lab.start_picture_;
		this.filename_=lab.filename_;
		
	}
//------------------------------------------------------------------------------
/**
 * Standard constructor
 * @param filename name of the XML file to load
 * @exception IllegalArgumentException if the specified file
 * can't be found
 */
  public LoadLabyrinth(String filename, URL document_base)
    throws IllegalArgumentException
  {
    document_base_ = document_base;
		filename_=filename;
		labyrinth_field_=null;
    try
    {
		// Load the ContentHandler ********************************************
		SAXParserFactory factory = SAXParserFactory.newInstance();
		factory.setValidating(true);   //neu
		SAXParser saxParser = factory.newSAXParser();
		XMLReader xmlReader = saxParser.getXMLReader(); 
		labyrinth_xml_content_handler_ = new LabyrinthXMLContentHandler();
		labyrinth_xml_content_handler_.setDocumentBase(document_base_);
			labyrinth_xml_content_handler_.setDocument(filename_);
		xmlReader.setContentHandler(labyrinth_xml_content_handler_);
			
		XMLErrorHandler xml_error_handler = new XMLErrorHandler();
		xmlReader.setErrorHandler(xml_error_handler);
			
		xmlReader.parse(filename_);
		// (END OF) Load the ContentHandler ********************************************

		// Just some variables ***********************************************************
		boolean is_load_labyrinth=false;
		char start_symbol;
		char wall_symbol;
		char diamond_symbol;
		char way_symbol;
		char zero_symbol;
		char one_symbol;
		char two_symbol;
		char three_symbol;
		int set_num_diamonds;
		int set_num_way_cells;
		int set_min_way_lenght;
		int set_max_way_lenght;
		int set_build_angles;
		int set_build_trees;
		int set_build_places;
		int set_build_loops;
		int rows = labyrinth_xml_content_handler_.getHeight();
		int cols = labyrinth_xml_content_handler_.getWidth();
		// (END OF) Just some variables *************************************************

		if(xml_error_handler.isXMLError()==false && labyrinth_xml_content_handler_.isXMLError()==false)
		{
			// This point IS reached at normal operation ************************************
			try
			{
				if(cols<=0)throw(new IllegalArgumentException("\"<" + TagName.WIDTH + ">\""+"must be > 0"));
				if(rows<=0)throw(new IllegalArgumentException("\"<" + TagName.HEIGHT + ">\""+"must be > 0"));	
				labyrinth_field_ = new LabyrinthField(rows, cols);
	
				max_row_=rows-1;
				max_col_=cols-1;
	
				String name = labyrinth_xml_content_handler_.getLevelName();
				labyrinth_field_.setName(name);
	
				if(labyrinth_xml_content_handler_.isInstructionList()==true)
				{
					/*SAXParserFactory factory = SAXParserFactory.newInstance();
					SAXParser saxParser = factory.newSAXParser();
					XMLReader xmlReader = saxParser.getXMLReader();
					String instruction_file_name = labyrinth_xml_content_handler_.getInstructionFileName();
					instruction_listxml_content_handler = new InstructionListXMLContentHandler();
					xmlReader.setContentHandler(instruction_listxml_content_handler);
					xmlReader.parse(instruction_file_name);
					labyrinth_field_.setInstructionList();*/
				}
	
				if(labyrinth_xml_content_handler_.isLoadLabyrinth()==true)
				{
					// Is not executed at normal operation, isLoadLabyrinth=false
					System.out.println("isLoadLabyrinth()");
					is_load_labyrinth=true;
					start_symbol = labyrinth_xml_content_handler_.getStartSymbol();
					wall_symbol = labyrinth_xml_content_handler_.getWallSymbol();
					diamond_symbol = labyrinth_xml_content_handler_.getDiamondSymbol();
					way_symbol = labyrinth_xml_content_handler_.getWaySymbol();
					zero_symbol = labyrinth_xml_content_handler_.getZeroSymbol();
					one_symbol = labyrinth_xml_content_handler_.getOneSymbol();
					two_symbol = labyrinth_xml_content_handler_.getTwoSymbol();
					three_symbol = labyrinth_xml_content_handler_.getThreeSymbol();
					char[][] labyrinth   = labyrinth_xml_content_handler_.getLabyrinth();
					Direction start_direction = labyrinth_xml_content_handler_.getStartingDirection();
					labyrinth_field_.setStartDirection(start_direction);
	
					char current_symbol;
					for (int row_count = 0; row_count < rows; row_count++)
					{
						for (int col_count = 0; col_count < cols; col_count++)
						{
							current_symbol = labyrinth[row_count][col_count];
							if(current_symbol == start_symbol)
							{
								//startpoint_row_=row_count;
								//startpoint_col_=col_count;
								//current_cell.convertToStartpoint();
								labyrinth_field_.convertToStartpoint(row_count,col_count);
							}
							else if(current_symbol == wall_symbol)
							{
								//current_cell.convertToWall();
								//labyrinth_field_.convertToWall(row_count,col_count);
							}
							else if(current_symbol == diamond_symbol)
							{
								//current_cell.convertToWay();
								//current_cell.setDiamond();
								labyrinth_field_.convertToWay(row_count,col_count);
								labyrinth_field_.setDiamond(row_count,col_count);
							}
							else if(current_symbol == way_symbol)
							{
								//current_cell.convertToWay();
								labyrinth_field_.convertToWay(row_count,col_count);
							}
							else if(current_symbol == zero_symbol)
							{
								labyrinth_field_.convertToWay(row_count,col_count);
								labyrinth_field_.setCellMark(row_count,col_count,(int) 0);
							}
							else if(current_symbol == one_symbol)
							{
								labyrinth_field_.convertToWay(row_count,col_count);
								labyrinth_field_.setCellMark(row_count,col_count,(int) 1);
							}
							else if(current_symbol == two_symbol)
							{
								labyrinth_field_.convertToWay(row_count,col_count);
								labyrinth_field_.setCellMark(row_count,col_count,(int) 2);
							}
							else if(current_symbol == three_symbol)
							{
								labyrinth_field_.convertToWay(row_count,col_count);
								labyrinth_field_.setCellMark(row_count,col_count,(int) 3);
							}
						}
					}
				}
				
	
				
				if(labyrinth_xml_content_handler_.isCreateLabyrinth()==true)
				{
					// This piece of code is executed at normal operation, isCreateLabyrinth=true
					System.out.println("isCreateLabyrinth()");
					set_num_diamonds = labyrinth_xml_content_handler_.getNumDiamonds();
					set_num_way_cells = labyrinth_xml_content_handler_.getNumWayCells();
					set_min_way_lenght = labyrinth_xml_content_handler_.getMinWayLenght();
					set_max_way_lenght = labyrinth_xml_content_handler_.getMaxWayLenght();
					set_build_angles = labyrinth_xml_content_handler_.getBuildAngles();
					set_build_trees = labyrinth_xml_content_handler_.getBuildTrees();
					set_build_places = labyrinth_xml_content_handler_.getBuildPlaces();
					set_build_loops = labyrinth_xml_content_handler_.getBuildLoops();
	
					create_labyrinth_ = new CreateLabyrinth(labyrinth_field_);
					create_labyrinth_.setNumDiamonds(set_num_diamonds);
					create_labyrinth_.setNumWayCells(set_num_way_cells);
					create_labyrinth_.setMinWayLenght(set_min_way_lenght);
					create_labyrinth_.setMaxWayLenght(set_max_way_lenght);
					create_labyrinth_.setBuildAngles(set_build_angles);
					create_labyrinth_.setBuildTrees(set_build_trees);
					create_labyrinth_.setBuildPlaces(set_build_places);
					create_labyrinth_.setBuildLoops(set_build_loops);
	
					create_labyrinth_.calculateWays();
					//System.out.println("calculateWays()");
	
					if(set_num_diamonds!=0)
					{
						//System.out.println("setDiamonds()");
						create_labyrinth_.setDiamonds();
					}
					labyrinth_field_=create_labyrinth_.getLabyrinthField();
				}
	
				is_style_=labyrinth_xml_content_handler_.isStyle();
				}
			catch(IllegalArgumentException exc) // SECOND TRY ENDS HERE
			{
				labyrinth_field_=null;
				System.out.println("ERROR");
				URL new_url = new URL(document_base_,filename_);
			  System.out.println("In File "+new_url.toString()); 
				System.out.println(exc.getMessage());
			}
		}
	} // FIRST TRY ENDS HERE
	catch(SAXParseException exc)
	{
		System.out.println("ERROR");
		System.out.println("In File " + exc.getSystemId());
		System.out.println(exc.getMessage());
	}		
	catch(SAXException exc)
	{
		System.out.println("SAXException");
		System.err.println(exc.getMessage());
		System.err.println(exc.toString());
	}
	catch(ParserConfigurationException exc)
	{
		System.out.println("ParserConfigurationException");
		System.err.println(exc.getMessage());
	}
	catch(IOException exc)
	{
		System.out.println("IOException");
		System.err.println(exc.getMessage());
	}

  }

//------------------------------------------------------------------------------
/**
 *
 *
 */
  public LabyrinthField getLabyrinthField()
  {
    return labyrinth_field_;
  }



//------------------------------------------------------------------------------
/**
 * standard toString method for debugging
 * @return info about the labyrinth field in string-format
 */

  public String toString()
  {
    return("LoadLabyrinth: " +
           super.toString());
  }

  public boolean isStyle()
  {
    return is_style_;
  }

  public String getWallPicture()
  {
    return labyrinth_xml_content_handler_.getWallPicture();
  }

  public String getWayPicture()
  {
    return way_picture_ = labyrinth_xml_content_handler_.getWayPicture();
  }

  public String getRobotPicture()
  {
    return robot_picture_ = labyrinth_xml_content_handler_.getRobotPicture();
  }

  public String getDiamondPicture()
  {
    return diamond_picture_ = labyrinth_xml_content_handler_.getDiamondPicture();
  }

  public String getStartPicture()
  {
    return start_picture_ = labyrinth_xml_content_handler_.getStartPicture();
  }

}
